# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.workers.observability import (
    destination_list_params,
    destination_create_params,
    destination_update_params,
)
from ....types.workers.observability.destination_list_response import DestinationListResponse
from ....types.workers.observability.destination_create_response import DestinationCreateResponse
from ....types.workers.observability.destination_delete_response import DestinationDeleteResponse
from ....types.workers.observability.destination_update_response import DestinationUpdateResponse

__all__ = ["DestinationsResource", "AsyncDestinationsResource"]


class DestinationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DestinationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DestinationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DestinationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DestinationsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        configuration: destination_create_params.Configuration,
        enabled: bool,
        name: str,
        skip_preflight_check: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DestinationCreateResponse:
        """
        Create a new Workers Observability Telemetry Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/workers/observability/destinations", account_id=account_id),
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "enabled": enabled,
                    "name": name,
                    "skip_preflight_check": skip_preflight_check,
                },
                destination_create_params.DestinationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DestinationCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DestinationCreateResponse], ResultWrapper[DestinationCreateResponse]),
        )

    def update(
        self,
        slug: str,
        *,
        account_id: str,
        configuration: destination_update_params.Configuration,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DestinationUpdateResponse:
        """
        Update an existing Workers Observability Telemetry Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/workers/observability/destinations/{slug}", account_id=account_id, slug=slug
            ),
            body=maybe_transform(
                {
                    "configuration": configuration,
                    "enabled": enabled,
                },
                destination_update_params.DestinationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DestinationUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DestinationUpdateResponse], ResultWrapper[DestinationUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created", "updated"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[DestinationListResponse]:
        """
        List your Workers Observability Telemetry Destinations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workers/observability/destinations", account_id=account_id),
            page=SyncSinglePage[DestinationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "order": order,
                        "order_by": order_by,
                        "page": page,
                        "per_page": per_page,
                    },
                    destination_list_params.DestinationListParams,
                ),
            ),
            model=DestinationListResponse,
        )

    def delete(
        self,
        slug: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DestinationDeleteResponse]:
        """
        Delete a Workers Observability Telemetry Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workers/observability/destinations/{slug}", account_id=account_id, slug=slug
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DestinationDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DestinationDeleteResponse]], ResultWrapper[DestinationDeleteResponse]),
        )


class AsyncDestinationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDestinationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDestinationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDestinationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDestinationsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        configuration: destination_create_params.Configuration,
        enabled: bool,
        name: str,
        skip_preflight_check: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DestinationCreateResponse:
        """
        Create a new Workers Observability Telemetry Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/workers/observability/destinations", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "enabled": enabled,
                    "name": name,
                    "skip_preflight_check": skip_preflight_check,
                },
                destination_create_params.DestinationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DestinationCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DestinationCreateResponse], ResultWrapper[DestinationCreateResponse]),
        )

    async def update(
        self,
        slug: str,
        *,
        account_id: str,
        configuration: destination_update_params.Configuration,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DestinationUpdateResponse:
        """
        Update an existing Workers Observability Telemetry Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/workers/observability/destinations/{slug}", account_id=account_id, slug=slug
            ),
            body=await async_maybe_transform(
                {
                    "configuration": configuration,
                    "enabled": enabled,
                },
                destination_update_params.DestinationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[DestinationUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[DestinationUpdateResponse], ResultWrapper[DestinationUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created", "updated"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DestinationListResponse, AsyncSinglePage[DestinationListResponse]]:
        """
        List your Workers Observability Telemetry Destinations.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workers/observability/destinations", account_id=account_id),
            page=AsyncSinglePage[DestinationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "order": order,
                        "order_by": order_by,
                        "page": page,
                        "per_page": per_page,
                    },
                    destination_list_params.DestinationListParams,
                ),
            ),
            model=DestinationListResponse,
        )

    async def delete(
        self,
        slug: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DestinationDeleteResponse]:
        """
        Delete a Workers Observability Telemetry Destination.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workers/observability/destinations/{slug}", account_id=account_id, slug=slug
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[DestinationDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DestinationDeleteResponse]], ResultWrapper[DestinationDeleteResponse]),
        )


class DestinationsResourceWithRawResponse:
    def __init__(self, destinations: DestinationsResource) -> None:
        self._destinations = destinations

        self.create = to_raw_response_wrapper(
            destinations.create,
        )
        self.update = to_raw_response_wrapper(
            destinations.update,
        )
        self.list = to_raw_response_wrapper(
            destinations.list,
        )
        self.delete = to_raw_response_wrapper(
            destinations.delete,
        )


class AsyncDestinationsResourceWithRawResponse:
    def __init__(self, destinations: AsyncDestinationsResource) -> None:
        self._destinations = destinations

        self.create = async_to_raw_response_wrapper(
            destinations.create,
        )
        self.update = async_to_raw_response_wrapper(
            destinations.update,
        )
        self.list = async_to_raw_response_wrapper(
            destinations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            destinations.delete,
        )


class DestinationsResourceWithStreamingResponse:
    def __init__(self, destinations: DestinationsResource) -> None:
        self._destinations = destinations

        self.create = to_streamed_response_wrapper(
            destinations.create,
        )
        self.update = to_streamed_response_wrapper(
            destinations.update,
        )
        self.list = to_streamed_response_wrapper(
            destinations.list,
        )
        self.delete = to_streamed_response_wrapper(
            destinations.delete,
        )


class AsyncDestinationsResourceWithStreamingResponse:
    def __init__(self, destinations: AsyncDestinationsResource) -> None:
        self._destinations = destinations

        self.create = async_to_streamed_response_wrapper(
            destinations.create,
        )
        self.update = async_to_streamed_response_wrapper(
            destinations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            destinations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            destinations.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["GSuiteGroupRule", "GSuite"]


class GSuite(BaseModel):
    email: str
    """The email of the Google Workspace group."""

    identity_provider_id: str
    """The ID of your Google Workspace identity provider."""


class GSuiteGroupRule(BaseModel):
    """
    Matches a group in Google Workspace.
    Requires a Google Workspace identity provider.
    """

    gsuite: GSuite



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PoolListParams"]


class PoolListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    monitor: str
    """
    The ID of the Monitor to use for checking the health of origins within this
    pool.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["DatasetListResponse", "Dataset"]


class Dataset(BaseModel):
    id: int

    description: str

    meta: object

    tags: List[str]

    title: str

    type: str


class DatasetListResponse(BaseModel):
    datasets: List[Dataset]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tail import (
    TailResource,
    AsyncTailResource,
    TailResourceWithRawResponse,
    AsyncTailResourceWithRawResponse,
    TailResourceWithStreamingResponse,
    AsyncTailResourceWithStreamingResponse,
)
from .assets import (
    AssetsResource,
    AsyncAssetsResource,
    AssetsResourceWithRawResponse,
    AsyncAssetsResourceWithRawResponse,
    AssetsResourceWithStreamingResponse,
    AsyncAssetsResourceWithStreamingResponse,
)
from .content import (
    ContentResource,
    AsyncContentResource,
    ContentResourceWithRawResponse,
    AsyncContentResourceWithRawResponse,
    ContentResourceWithStreamingResponse,
    AsyncContentResourceWithStreamingResponse,
)
from .scripts import (
    ScriptsResource,
    AsyncScriptsResource,
    ScriptsResourceWithRawResponse,
    AsyncScriptsResourceWithRawResponse,
    ScriptsResourceWithStreamingResponse,
    AsyncScriptsResourceWithStreamingResponse,
)
from .secrets import (
    SecretsResource,
    AsyncSecretsResource,
    SecretsResourceWithRawResponse,
    AsyncSecretsResourceWithRawResponse,
    SecretsResourceWithStreamingResponse,
    AsyncSecretsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)
from .schedules import (
    SchedulesResource,
    AsyncSchedulesResource,
    SchedulesResourceWithRawResponse,
    AsyncSchedulesResourceWithRawResponse,
    SchedulesResourceWithStreamingResponse,
    AsyncSchedulesResourceWithStreamingResponse,
)
from .subdomain import (
    SubdomainResource,
    AsyncSubdomainResource,
    SubdomainResourceWithRawResponse,
    AsyncSubdomainResourceWithRawResponse,
    SubdomainResourceWithStreamingResponse,
    AsyncSubdomainResourceWithStreamingResponse,
)
from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)
from .script_and_version_settings import (
    ScriptAndVersionSettingsResource,
    AsyncScriptAndVersionSettingsResource,
    ScriptAndVersionSettingsResourceWithRawResponse,
    AsyncScriptAndVersionSettingsResourceWithRawResponse,
    ScriptAndVersionSettingsResourceWithStreamingResponse,
    AsyncScriptAndVersionSettingsResourceWithStreamingResponse,
)

__all__ = [
    "AssetsResource",
    "AsyncAssetsResource",
    "AssetsResourceWithRawResponse",
    "AsyncAssetsResourceWithRawResponse",
    "AssetsResourceWithStreamingResponse",
    "AsyncAssetsResourceWithStreamingResponse",
    "SubdomainResource",
    "AsyncSubdomainResource",
    "SubdomainResourceWithRawResponse",
    "AsyncSubdomainResourceWithRawResponse",
    "SubdomainResourceWithStreamingResponse",
    "AsyncSubdomainResourceWithStreamingResponse",
    "SchedulesResource",
    "AsyncSchedulesResource",
    "SchedulesResourceWithRawResponse",
    "AsyncSchedulesResourceWithRawResponse",
    "SchedulesResourceWithStreamingResponse",
    "AsyncSchedulesResourceWithStreamingResponse",
    "TailResource",
    "AsyncTailResource",
    "TailResourceWithRawResponse",
    "AsyncTailResourceWithRawResponse",
    "TailResourceWithStreamingResponse",
    "AsyncTailResourceWithStreamingResponse",
    "ContentResource",
    "AsyncContentResource",
    "ContentResourceWithRawResponse",
    "AsyncContentResourceWithRawResponse",
    "ContentResourceWithStreamingResponse",
    "AsyncContentResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "DeploymentsResource",
    "AsyncDeploymentsResource",
    "DeploymentsResourceWithRawResponse",
    "AsyncDeploymentsResourceWithRawResponse",
    "DeploymentsResourceWithStreamingResponse",
    "AsyncDeploymentsResourceWithStreamingResponse",
    "VersionsResource",
    "AsyncVersionsResource",
    "VersionsResourceWithRawResponse",
    "AsyncVersionsResourceWithRawResponse",
    "VersionsResourceWithStreamingResponse",
    "AsyncVersionsResourceWithStreamingResponse",
    "SecretsResource",
    "AsyncSecretsResource",
    "SecretsResourceWithRawResponse",
    "AsyncSecretsResourceWithRawResponse",
    "SecretsResourceWithStreamingResponse",
    "AsyncSecretsResourceWithStreamingResponse",
    "ScriptAndVersionSettingsResource",
    "AsyncScriptAndVersionSettingsResource",
    "ScriptAndVersionSettingsResourceWithRawResponse",
    "AsyncScriptAndVersionSettingsResourceWithRawResponse",
    "ScriptAndVersionSettingsResourceWithStreamingResponse",
    "AsyncScriptAndVersionSettingsResourceWithStreamingResponse",
    "ScriptsResource",
    "AsyncScriptsResource",
    "ScriptsResourceWithRawResponse",
    "AsyncScriptsResourceWithRawResponse",
    "ScriptsResourceWithStreamingResponse",
    "AsyncScriptsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from .acls import (
    ACLsResource,
    AsyncACLsResource,
    ACLsResourceWithRawResponse,
    AsyncACLsResourceWithRawResponse,
    ACLsResourceWithStreamingResponse,
    AsyncACLsResourceWithStreamingResponse,
)
from .lans import (
    LANsResource,
    AsyncLANsResource,
    LANsResourceWithRawResponse,
    AsyncLANsResourceWithRawResponse,
    LANsResourceWithStreamingResponse,
    AsyncLANsResourceWithStreamingResponse,
)
from .wans import (
    WANsResource,
    AsyncWANsResource,
    WANsResourceWithRawResponse,
    AsyncWANsResourceWithRawResponse,
    WANsResourceWithStreamingResponse,
    AsyncWANsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import is_given, path_template, maybe_transform, strip_not_given, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_transit import (
    site_edit_params,
    site_list_params,
    site_create_params,
    site_update_params,
)
from ....types.magic_transit.site import Site
from ....types.magic_transit.site_location_param import SiteLocationParam

__all__ = ["SitesResource", "AsyncSitesResource"]


class SitesResource(SyncAPIResource):
    @cached_property
    def acls(self) -> ACLsResource:
        return ACLsResource(self._client)

    @cached_property
    def lans(self) -> LANsResource:
        return LANsResource(self._client)

    @cached_property
    def wans(self) -> WANsResource:
        return WANsResource(self._client)

    @cached_property
    def with_raw_response(self) -> SitesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SitesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SitesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SitesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        connector_id: str | Omit = omit,
        description: str | Omit = omit,
        ha_mode: bool | Omit = omit,
        location: SiteLocationParam | Omit = omit,
        secondary_connector_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Creates a new Site

        Args:
          account_id: Identifier

          name: The name of the site.

          connector_id: Magic Connector identifier tag.

          ha_mode: Site high availability mode. If set to true, the site can have two connectors
              and runs in high availability mode.

          location: Location of site in latitude and longitude.

          secondary_connector_id: Magic Connector identifier tag. Used when high availability mode is on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/magic/sites", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "connector_id": connector_id,
                    "description": description,
                    "ha_mode": ha_mode,
                    "location": location,
                    "secondary_connector_id": secondary_connector_id,
                },
                site_create_params.SiteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    def update(
        self,
        site_id: str,
        *,
        account_id: str,
        connector_id: str | Omit = omit,
        description: str | Omit = omit,
        location: SiteLocationParam | Omit = omit,
        name: str | Omit = omit,
        secondary_connector_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Update a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          connector_id: Magic Connector identifier tag.

          location: Location of site in latitude and longitude.

          name: The name of the site.

          secondary_connector_id: Magic Connector identifier tag. Used when high availability mode is on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            body=maybe_transform(
                {
                    "connector_id": connector_id,
                    "description": description,
                    "location": location,
                    "name": name,
                    "secondary_connector_id": secondary_connector_id,
                },
                site_update_params.SiteUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    def list(
        self,
        *,
        account_id: str,
        connectorid: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Site]:
        """Lists Sites associated with an account.

        Use connectorid query param to return
        sites where connectorid matches either site.ConnectorID or
        site.SecondaryConnectorID.

        Args:
          account_id: Identifier

          connectorid: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites", account_id=account_id),
            page=SyncSinglePage[Site],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"connectorid": connectorid}, site_list_params.SiteListParams),
            ),
            model=Site,
        )

    def delete(
        self,
        site_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Remove a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    def edit(
        self,
        site_id: str,
        *,
        account_id: str,
        connector_id: str | Omit = omit,
        description: str | Omit = omit,
        location: SiteLocationParam | Omit = omit,
        name: str | Omit = omit,
        secondary_connector_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Patch a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          connector_id: Magic Connector identifier tag.

          location: Location of site in latitude and longitude.

          name: The name of the site.

          secondary_connector_id: Magic Connector identifier tag. Used when high availability mode is on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            body=maybe_transform(
                {
                    "connector_id": connector_id,
                    "description": description,
                    "location": location,
                    "name": name,
                    "secondary_connector_id": secondary_connector_id,
                },
                site_edit_params.SiteEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    def get(
        self,
        site_id: str,
        *,
        account_id: str,
        x_magic_new_hc_target: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Get a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        extra_headers = {
            **strip_not_given(
                {
                    "x-magic-new-hc-target": ("true" if x_magic_new_hc_target else "false")
                    if is_given(x_magic_new_hc_target)
                    else not_given
                }
            ),
            **(extra_headers or {}),
        }
        return self._get(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )


class AsyncSitesResource(AsyncAPIResource):
    @cached_property
    def acls(self) -> AsyncACLsResource:
        return AsyncACLsResource(self._client)

    @cached_property
    def lans(self) -> AsyncLANsResource:
        return AsyncLANsResource(self._client)

    @cached_property
    def wans(self) -> AsyncWANsResource:
        return AsyncWANsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSitesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSitesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSitesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSitesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        connector_id: str | Omit = omit,
        description: str | Omit = omit,
        ha_mode: bool | Omit = omit,
        location: SiteLocationParam | Omit = omit,
        secondary_connector_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Creates a new Site

        Args:
          account_id: Identifier

          name: The name of the site.

          connector_id: Magic Connector identifier tag.

          ha_mode: Site high availability mode. If set to true, the site can have two connectors
              and runs in high availability mode.

          location: Location of site in latitude and longitude.

          secondary_connector_id: Magic Connector identifier tag. Used when high availability mode is on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/magic/sites", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "connector_id": connector_id,
                    "description": description,
                    "ha_mode": ha_mode,
                    "location": location,
                    "secondary_connector_id": secondary_connector_id,
                },
                site_create_params.SiteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    async def update(
        self,
        site_id: str,
        *,
        account_id: str,
        connector_id: str | Omit = omit,
        description: str | Omit = omit,
        location: SiteLocationParam | Omit = omit,
        name: str | Omit = omit,
        secondary_connector_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Update a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          connector_id: Magic Connector identifier tag.

          location: Location of site in latitude and longitude.

          name: The name of the site.

          secondary_connector_id: Magic Connector identifier tag. Used when high availability mode is on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            body=await async_maybe_transform(
                {
                    "connector_id": connector_id,
                    "description": description,
                    "location": location,
                    "name": name,
                    "secondary_connector_id": secondary_connector_id,
                },
                site_update_params.SiteUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    def list(
        self,
        *,
        account_id: str,
        connectorid: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Site, AsyncSinglePage[Site]]:
        """Lists Sites associated with an account.

        Use connectorid query param to return
        sites where connectorid matches either site.ConnectorID or
        site.SecondaryConnectorID.

        Args:
          account_id: Identifier

          connectorid: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/sites", account_id=account_id),
            page=AsyncSinglePage[Site],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"connectorid": connectorid}, site_list_params.SiteListParams),
            ),
            model=Site,
        )

    async def delete(
        self,
        site_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Remove a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    async def edit(
        self,
        site_id: str,
        *,
        account_id: str,
        connector_id: str | Omit = omit,
        description: str | Omit = omit,
        location: SiteLocationParam | Omit = omit,
        name: str | Omit = omit,
        secondary_connector_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Patch a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          connector_id: Magic Connector identifier tag.

          location: Location of site in latitude and longitude.

          name: The name of the site.

          secondary_connector_id: Magic Connector identifier tag. Used when high availability mode is on.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            body=await async_maybe_transform(
                {
                    "connector_id": connector_id,
                    "description": description,
                    "location": location,
                    "name": name,
                    "secondary_connector_id": secondary_connector_id,
                },
                site_edit_params.SiteEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )

    async def get(
        self,
        site_id: str,
        *,
        account_id: str,
        x_magic_new_hc_target: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Site:
        """
        Get a specific Site.

        Args:
          account_id: Identifier

          site_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not site_id:
            raise ValueError(f"Expected a non-empty value for `site_id` but received {site_id!r}")
        extra_headers = {
            **strip_not_given(
                {
                    "x-magic-new-hc-target": ("true" if x_magic_new_hc_target else "false")
                    if is_given(x_magic_new_hc_target)
                    else not_given
                }
            ),
            **(extra_headers or {}),
        }
        return await self._get(
            path_template("/accounts/{account_id}/magic/sites/{site_id}", account_id=account_id, site_id=site_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Site]._unwrapper,
            ),
            cast_to=cast(Type[Site], ResultWrapper[Site]),
        )


class SitesResourceWithRawResponse:
    def __init__(self, sites: SitesResource) -> None:
        self._sites = sites

        self.create = to_raw_response_wrapper(
            sites.create,
        )
        self.update = to_raw_response_wrapper(
            sites.update,
        )
        self.list = to_raw_response_wrapper(
            sites.list,
        )
        self.delete = to_raw_response_wrapper(
            sites.delete,
        )
        self.edit = to_raw_response_wrapper(
            sites.edit,
        )
        self.get = to_raw_response_wrapper(
            sites.get,
        )

    @cached_property
    def acls(self) -> ACLsResourceWithRawResponse:
        return ACLsResourceWithRawResponse(self._sites.acls)

    @cached_property
    def lans(self) -> LANsResourceWithRawResponse:
        return LANsResourceWithRawResponse(self._sites.lans)

    @cached_property
    def wans(self) -> WANsResourceWithRawResponse:
        return WANsResourceWithRawResponse(self._sites.wans)


class AsyncSitesResourceWithRawResponse:
    def __init__(self, sites: AsyncSitesResource) -> None:
        self._sites = sites

        self.create = async_to_raw_response_wrapper(
            sites.create,
        )
        self.update = async_to_raw_response_wrapper(
            sites.update,
        )
        self.list = async_to_raw_response_wrapper(
            sites.list,
        )
        self.delete = async_to_raw_response_wrapper(
            sites.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            sites.edit,
        )
        self.get = async_to_raw_response_wrapper(
            sites.get,
        )

    @cached_property
    def acls(self) -> AsyncACLsResourceWithRawResponse:
        return AsyncACLsResourceWithRawResponse(self._sites.acls)

    @cached_property
    def lans(self) -> AsyncLANsResourceWithRawResponse:
        return AsyncLANsResourceWithRawResponse(self._sites.lans)

    @cached_property
    def wans(self) -> AsyncWANsResourceWithRawResponse:
        return AsyncWANsResourceWithRawResponse(self._sites.wans)


class SitesResourceWithStreamingResponse:
    def __init__(self, sites: SitesResource) -> None:
        self._sites = sites

        self.create = to_streamed_response_wrapper(
            sites.create,
        )
        self.update = to_streamed_response_wrapper(
            sites.update,
        )
        self.list = to_streamed_response_wrapper(
            sites.list,
        )
        self.delete = to_streamed_response_wrapper(
            sites.delete,
        )
        self.edit = to_streamed_response_wrapper(
            sites.edit,
        )
        self.get = to_streamed_response_wrapper(
            sites.get,
        )

    @cached_property
    def acls(self) -> ACLsResourceWithStreamingResponse:
        return ACLsResourceWithStreamingResponse(self._sites.acls)

    @cached_property
    def lans(self) -> LANsResourceWithStreamingResponse:
        return LANsResourceWithStreamingResponse(self._sites.lans)

    @cached_property
    def wans(self) -> WANsResourceWithStreamingResponse:
        return WANsResourceWithStreamingResponse(self._sites.wans)


class AsyncSitesResourceWithStreamingResponse:
    def __init__(self, sites: AsyncSitesResource) -> None:
        self._sites = sites

        self.create = async_to_streamed_response_wrapper(
            sites.create,
        )
        self.update = async_to_streamed_response_wrapper(
            sites.update,
        )
        self.list = async_to_streamed_response_wrapper(
            sites.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            sites.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            sites.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            sites.get,
        )

    @cached_property
    def acls(self) -> AsyncACLsResourceWithStreamingResponse:
        return AsyncACLsResourceWithStreamingResponse(self._sites.acls)

    @cached_property
    def lans(self) -> AsyncLANsResourceWithStreamingResponse:
        return AsyncLANsResourceWithStreamingResponse(self._sites.lans)

    @cached_property
    def wans(self) -> AsyncWANsResourceWithStreamingResponse:
        return AsyncWANsResourceWithStreamingResponse(self._sites.wans)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["FallbackOriginGetResponse"]


class FallbackOriginGetResponse(BaseModel):
    created_at: Optional[datetime] = None
    """This is the time the fallback origin was created."""

    errors: Optional[List[str]] = None
    """
    These are errors that were encountered while trying to activate a fallback
    origin.
    """

    origin: Optional[str] = None
    """Your origin hostname that requests to your custom hostnames will be sent to."""

    status: Optional[
        Literal[
            "initializing",
            "pending_deployment",
            "pending_deletion",
            "active",
            "deployment_timed_out",
            "deletion_timed_out",
        ]
    ] = None
    """Status of the fallback origin's activation."""

    updated_at: Optional[datetime] = None
    """This is the time the fallback origin was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ScriptListParams"]


class ScriptListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier"""

    direction: Literal["asc", "desc"]
    """The direction used to sort returned scripts."""

    exclude_cdn_cgi: bool
    """When true, excludes scripts seen in a `/cdn-cgi` path from the returned scripts.

    The default value is true.
    """

    exclude_duplicates: bool
    """When true, excludes duplicate scripts.

    We consider a script duplicate of another if their javascript content matches
    and they share the same url host and zone hostname. In such case, we return the
    most recent script for the URL host and zone hostname combination.
    """

    exclude_urls: str
    """
    Excludes scripts whose URL contains one of the URL-encoded URLs separated by
    commas.
    """

    export: Literal["csv"]
    """Export the list of scripts as a file, limited to 50000 entries."""

    hosts: str
    """
    Includes scripts that match one or more URL-encoded hostnames separated by
    commas.

    Wildcards are supported at the start and end of each hostname to support starts
    with, ends with and contains. If no wildcards are used, results will be filtered
    by exact match
    """

    order_by: Literal["first_seen_at", "last_seen_at"]
    """The field used to sort returned scripts."""

    page: str
    """The current page number of the paginated results.

    We additionally support a special value "all". When "all" is used, the API will
    return all the scripts with the applied filters in a single page. This feature
    is best-effort and it may only work for zones with a low number of scripts
    """

    page_url: str
    """
    Includes scripts that match one or more page URLs (separated by commas) where
    they were last seen

    Wildcards are supported at the start and end of each page URL to support starts
    with, ends with and contains. If no wildcards are used, results will be filtered
    by exact match
    """

    per_page: float
    """The number of results per page."""

    prioritize_malicious: bool
    """When true, malicious scripts appear first in the returned scripts."""

    status: str
    """Filters the returned scripts using a comma-separated list of scripts statuses.

    Accepted values: `active`, `infrequent`, and `inactive`. The default value is
    `active`.
    """

    urls: str
    """
    Includes scripts whose URL contain one or more URL-encoded URLs separated by
    commas.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["OperationDeleteResponse"]


class OperationDeleteResponse(BaseModel):
    operation_id: Optional[str] = None
    """UUID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .catch_all_action import CatchAllAction as CatchAllAction
from .catch_all_matcher import CatchAllMatcher as CatchAllMatcher
from .catch_all_action_param import CatchAllActionParam as CatchAllActionParam
from .catch_all_get_response import CatchAllGetResponse as CatchAllGetResponse
from .catch_all_matcher_param import CatchAllMatcherParam as CatchAllMatcherParam
from .catch_all_update_params import CatchAllUpdateParams as CatchAllUpdateParams
from .catch_all_update_response import CatchAllUpdateResponse as CatchAllUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from .versions import (
    VersionsResource,
    AsyncVersionsResource,
    VersionsResourceWithRawResponse,
    AsyncVersionsResourceWithRawResponse,
    VersionsResourceWithStreamingResponse,
    AsyncVersionsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.workflows import workflow_list_params, workflow_update_params
from .instances.instances import (
    InstancesResource,
    AsyncInstancesResource,
    InstancesResourceWithRawResponse,
    AsyncInstancesResourceWithRawResponse,
    InstancesResourceWithStreamingResponse,
    AsyncInstancesResourceWithStreamingResponse,
)
from ...types.workflows.workflow_get_response import WorkflowGetResponse
from ...types.workflows.workflow_list_response import WorkflowListResponse
from ...types.workflows.workflow_delete_response import WorkflowDeleteResponse
from ...types.workflows.workflow_update_response import WorkflowUpdateResponse

__all__ = ["WorkflowsResource", "AsyncWorkflowsResource"]


class WorkflowsResource(SyncAPIResource):
    @cached_property
    def instances(self) -> InstancesResource:
        return InstancesResource(self._client)

    @cached_property
    def versions(self) -> VersionsResource:
        return VersionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> WorkflowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return WorkflowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WorkflowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return WorkflowsResourceWithStreamingResponse(self)

    def update(
        self,
        workflow_name: str,
        *,
        account_id: str,
        class_name: str,
        script_name: str,
        limits: workflow_update_params.Limits | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowUpdateResponse:
        """
        Creates a new workflow or updates an existing workflow definition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}", account_id=account_id, workflow_name=workflow_name
            ),
            body=maybe_transform(
                {
                    "class_name": class_name,
                    "script_name": script_name,
                    "limits": limits,
                },
                workflow_update_params.WorkflowUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowUpdateResponse], ResultWrapper[WorkflowUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[WorkflowListResponse]:
        """
        Lists all workflows configured for the account.

        Args:
          search: Allows filtering workflows` name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workflows", account_id=account_id),
            page=SyncV4PagePaginationArray[WorkflowListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    workflow_list_params.WorkflowListParams,
                ),
            ),
            model=WorkflowListResponse,
        )

    def delete(
        self,
        workflow_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowDeleteResponse:
        """Deletes a Workflow.

        This only deletes the Workflow and does not delete or modify
        any Worker associated to this Workflow or bounded to it.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}", account_id=account_id, workflow_name=workflow_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowDeleteResponse], ResultWrapper[WorkflowDeleteResponse]),
        )

    def get(
        self,
        workflow_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowGetResponse:
        """
        Retrieves configuration and metadata for a specific workflow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}", account_id=account_id, workflow_name=workflow_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowGetResponse], ResultWrapper[WorkflowGetResponse]),
        )


class AsyncWorkflowsResource(AsyncAPIResource):
    @cached_property
    def instances(self) -> AsyncInstancesResource:
        return AsyncInstancesResource(self._client)

    @cached_property
    def versions(self) -> AsyncVersionsResource:
        return AsyncVersionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWorkflowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWorkflowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWorkflowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncWorkflowsResourceWithStreamingResponse(self)

    async def update(
        self,
        workflow_name: str,
        *,
        account_id: str,
        class_name: str,
        script_name: str,
        limits: workflow_update_params.Limits | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowUpdateResponse:
        """
        Creates a new workflow or updates an existing workflow definition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}", account_id=account_id, workflow_name=workflow_name
            ),
            body=await async_maybe_transform(
                {
                    "class_name": class_name,
                    "script_name": script_name,
                    "limits": limits,
                },
                workflow_update_params.WorkflowUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowUpdateResponse], ResultWrapper[WorkflowUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[WorkflowListResponse, AsyncV4PagePaginationArray[WorkflowListResponse]]:
        """
        Lists all workflows configured for the account.

        Args:
          search: Allows filtering workflows` name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/workflows", account_id=account_id),
            page=AsyncV4PagePaginationArray[WorkflowListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    workflow_list_params.WorkflowListParams,
                ),
            ),
            model=WorkflowListResponse,
        )

    async def delete(
        self,
        workflow_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowDeleteResponse:
        """Deletes a Workflow.

        This only deletes the Workflow and does not delete or modify
        any Worker associated to this Workflow or bounded to it.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}", account_id=account_id, workflow_name=workflow_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowDeleteResponse], ResultWrapper[WorkflowDeleteResponse]),
        )

    async def get(
        self,
        workflow_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WorkflowGetResponse:
        """
        Retrieves configuration and metadata for a specific workflow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not workflow_name:
            raise ValueError(f"Expected a non-empty value for `workflow_name` but received {workflow_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workflows/{workflow_name}", account_id=account_id, workflow_name=workflow_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[WorkflowGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[WorkflowGetResponse], ResultWrapper[WorkflowGetResponse]),
        )


class WorkflowsResourceWithRawResponse:
    def __init__(self, workflows: WorkflowsResource) -> None:
        self._workflows = workflows

        self.update = to_raw_response_wrapper(
            workflows.update,
        )
        self.list = to_raw_response_wrapper(
            workflows.list,
        )
        self.delete = to_raw_response_wrapper(
            workflows.delete,
        )
        self.get = to_raw_response_wrapper(
            workflows.get,
        )

    @cached_property
    def instances(self) -> InstancesResourceWithRawResponse:
        return InstancesResourceWithRawResponse(self._workflows.instances)

    @cached_property
    def versions(self) -> VersionsResourceWithRawResponse:
        return VersionsResourceWithRawResponse(self._workflows.versions)


class AsyncWorkflowsResourceWithRawResponse:
    def __init__(self, workflows: AsyncWorkflowsResource) -> None:
        self._workflows = workflows

        self.update = async_to_raw_response_wrapper(
            workflows.update,
        )
        self.list = async_to_raw_response_wrapper(
            workflows.list,
        )
        self.delete = async_to_raw_response_wrapper(
            workflows.delete,
        )
        self.get = async_to_raw_response_wrapper(
            workflows.get,
        )

    @cached_property
    def instances(self) -> AsyncInstancesResourceWithRawResponse:
        return AsyncInstancesResourceWithRawResponse(self._workflows.instances)

    @cached_property
    def versions(self) -> AsyncVersionsResourceWithRawResponse:
        return AsyncVersionsResourceWithRawResponse(self._workflows.versions)


class WorkflowsResourceWithStreamingResponse:
    def __init__(self, workflows: WorkflowsResource) -> None:
        self._workflows = workflows

        self.update = to_streamed_response_wrapper(
            workflows.update,
        )
        self.list = to_streamed_response_wrapper(
            workflows.list,
        )
        self.delete = to_streamed_response_wrapper(
            workflows.delete,
        )
        self.get = to_streamed_response_wrapper(
            workflows.get,
        )

    @cached_property
    def instances(self) -> InstancesResourceWithStreamingResponse:
        return InstancesResourceWithStreamingResponse(self._workflows.instances)

    @cached_property
    def versions(self) -> VersionsResourceWithStreamingResponse:
        return VersionsResourceWithStreamingResponse(self._workflows.versions)


class AsyncWorkflowsResourceWithStreamingResponse:
    def __init__(self, workflows: AsyncWorkflowsResource) -> None:
        self._workflows = workflows

        self.update = async_to_streamed_response_wrapper(
            workflows.update,
        )
        self.list = async_to_streamed_response_wrapper(
            workflows.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            workflows.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            workflows.get,
        )

    @cached_property
    def instances(self) -> AsyncInstancesResourceWithStreamingResponse:
        return AsyncInstancesResourceWithStreamingResponse(self._workflows.instances)

    @cached_property
    def versions(self) -> AsyncVersionsResourceWithStreamingResponse:
        return AsyncVersionsResourceWithStreamingResponse(self._workflows.versions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ServiceListResponse"]


class ServiceListResponse(BaseModel):
    id: Optional[str] = None
    """Identifier of a Service on the Cloudflare network.

    Available services and their IDs may be found in the **List Services** endpoint.
    """

    name: Optional[str] = None
    """Name of a service running on the Cloudflare network"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["MiscategorizationCreateParams"]


class MiscategorizationCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    content_adds: Iterable[int]
    """Content category IDs to add."""

    content_removes: Iterable[int]
    """Content category IDs to remove."""

    indicator_type: Literal["domain", "ipv4", "ipv6", "url"]

    ip: Optional[str]
    """Provide only if indicator_type is `ipv4` or `ipv6`."""

    security_adds: Iterable[int]
    """Security category IDs to add."""

    security_removes: Iterable[int]
    """Security category IDs to remove."""

    url: str
    """Provide only if indicator_type is `domain` or `url`.

    Example if indicator_type is `domain`: `example.com`. Example if indicator_type
    is `url`: `https://example.com/news/`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .override import Override as Override
from .waf_rule import WAFRule as WAFRule
from .override_url import OverrideURL as OverrideURL
from .rewrite_action import RewriteAction as RewriteAction
from .waf_rule_param import WAFRuleParam as WAFRuleParam
from .package_list_params import PackageListParams as PackageListParams
from .override_list_params import OverrideListParams as OverrideListParams
from .package_get_response import PackageGetResponse as PackageGetResponse
from .rewrite_action_param import RewriteActionParam as RewriteActionParam
from .override_create_params import OverrideCreateParams as OverrideCreateParams
from .override_update_params import OverrideUpdateParams as OverrideUpdateParams
from .override_delete_response import OverrideDeleteResponse as OverrideDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .logo_get_params import LogoGetParams as LogoGetParams
from .match_get_params import MatchGetParams as MatchGetParams
from .query_get_params import QueryGetParams as QueryGetParams
from .logo_get_response import LogoGetResponse as LogoGetResponse
from .logo_create_params import LogoCreateParams as LogoCreateParams
from .match_get_response import MatchGetResponse as MatchGetResponse
from .query_get_response import QueryGetResponse as QueryGetResponse
from .logo_create_response import LogoCreateResponse as LogoCreateResponse
from .logo_delete_response import LogoDeleteResponse as LogoDeleteResponse
from .logo_match_get_params import LogoMatchGetParams as LogoMatchGetParams
from .logo_match_get_response import LogoMatchGetResponse as LogoMatchGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ip_get_params import IPGetParams as IPGetParams
from .network_create_params import NetworkCreateParams as NetworkCreateParams
from .network_delete_params import NetworkDeleteParams as NetworkDeleteParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["MeetingUpdateMeetingByIDParams", "AIConfig", "AIConfigSummarization", "AIConfigTranscription"]


class MeetingUpdateMeetingByIDParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]
    """The app identifier tag."""

    ai_config: AIConfig
    """
    The AI Config allows you to customize the behavior of meeting transcriptions and
    summaries
    """

    live_stream_on_start: bool
    """Specifies if the meeting should start getting livestreamed on start."""

    persist_chat: bool
    """
    If a meeting is updated to persist_chat, meeting chat would remain for a week
    within the meeting space.
    """

    record_on_start: bool
    """
    Specifies if the meeting should start getting recorded as soon as someone joins
    the meeting.
    """

    session_keep_alive_time_in_secs: float
    """
    Time in seconds, for which a session remains active, after the last participant
    has left the meeting.
    """

    status: Literal["ACTIVE", "INACTIVE"]
    """Whether the meeting is `ACTIVE` or `INACTIVE`.

    Users will not be able to join an `INACTIVE` meeting.
    """

    summarize_on_end: bool
    """Automatically generate summary of meetings using transcripts.

    Requires Transcriptions to be enabled, and can be retrieved via Webhooks or
    summary API.
    """

    title: str
    """Title of the meeting"""


class AIConfigSummarization(TypedDict, total=False):
    """Summary Config"""

    summary_type: Literal[
        "general",
        "team_meeting",
        "sales_call",
        "client_check_in",
        "interview",
        "daily_standup",
        "one_on_one_meeting",
        "lecture",
        "code_review",
    ]
    """Defines the style of the summary, such as general, team meeting, or sales call."""

    text_format: Literal["plain_text", "markdown"]
    """Determines the text format of the summary, such as plain text or markdown."""

    word_limit: int
    """Sets the maximum number of words in the meeting summary."""


class AIConfigTranscription(TypedDict, total=False):
    """Transcription Configurations"""

    keywords: SequenceNotStr[str]
    """Adds specific terms to improve accurate detection during transcription."""

    language: Literal["en-US", "en-IN", "de", "hi", "sv", "ru", "pl", "el", "fr", "nl"]
    """Specifies the language code for transcription to ensure accurate results."""

    profanity_filter: bool
    """Control the inclusion of offensive language in transcriptions."""


class AIConfig(TypedDict, total=False):
    """
    The AI Config allows you to customize the behavior of meeting transcriptions and summaries
    """

    summarization: AIConfigSummarization
    """Summary Config"""

    transcription: AIConfigTranscription
    """Transcription Configurations"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from .results import (
    ResultsResource,
    AsyncResultsResource,
    ResultsResourceWithRawResponse,
    AsyncResultsResourceWithRawResponse,
    ResultsResourceWithStreamingResponse,
    AsyncResultsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ScansResource", "AsyncScansResource"]


class ScansResource(SyncAPIResource):
    @cached_property
    def results(self) -> ResultsResource:
        return ResultsResource(self._client)

    @cached_property
    def config(self) -> ConfigResource:
        return ConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> ScansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ScansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ScansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ScansResourceWithStreamingResponse(self)


class AsyncScansResource(AsyncAPIResource):
    @cached_property
    def results(self) -> AsyncResultsResource:
        return AsyncResultsResource(self._client)

    @cached_property
    def config(self) -> AsyncConfigResource:
        return AsyncConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncScansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncScansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncScansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncScansResourceWithStreamingResponse(self)


class ScansResourceWithRawResponse:
    def __init__(self, scans: ScansResource) -> None:
        self._scans = scans

    @cached_property
    def results(self) -> ResultsResourceWithRawResponse:
        return ResultsResourceWithRawResponse(self._scans.results)

    @cached_property
    def config(self) -> ConfigResourceWithRawResponse:
        return ConfigResourceWithRawResponse(self._scans.config)


class AsyncScansResourceWithRawResponse:
    def __init__(self, scans: AsyncScansResource) -> None:
        self._scans = scans

    @cached_property
    def results(self) -> AsyncResultsResourceWithRawResponse:
        return AsyncResultsResourceWithRawResponse(self._scans.results)

    @cached_property
    def config(self) -> AsyncConfigResourceWithRawResponse:
        return AsyncConfigResourceWithRawResponse(self._scans.config)


class ScansResourceWithStreamingResponse:
    def __init__(self, scans: ScansResource) -> None:
        self._scans = scans

    @cached_property
    def results(self) -> ResultsResourceWithStreamingResponse:
        return ResultsResourceWithStreamingResponse(self._scans.results)

    @cached_property
    def config(self) -> ConfigResourceWithStreamingResponse:
        return ConfigResourceWithStreamingResponse(self._scans.config)


class AsyncScansResourceWithStreamingResponse:
    def __init__(self, scans: AsyncScansResource) -> None:
        self._scans = scans

    @cached_property
    def results(self) -> AsyncResultsResourceWithStreamingResponse:
        return AsyncResultsResourceWithStreamingResponse(self._scans.results)

    @cached_property
    def config(self) -> AsyncConfigResourceWithStreamingResponse:
        return AsyncConfigResourceWithStreamingResponse(self._scans.config)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ...._types import (
    Body,
    Omit,
    Query,
    Headers,
    NotGiven,
    FileTypes,
    SequenceNotStr,
    omit,
    not_given,
)
from ...._utils import is_given, path_template, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncV4PagePagination, AsyncV4PagePagination
from ...._base_client import AsyncPaginator, make_request_options
from ....types.workers.scripts import version_list_params, version_create_params
from ....types.workers.scripts.version_get_response import VersionGetResponse
from ....types.workers.scripts.version_list_response import VersionListResponse
from ....types.workers.scripts.version_create_response import VersionCreateResponse

__all__ = ["VersionsResource", "AsyncVersionsResource"]


class VersionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return VersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return VersionsResourceWithStreamingResponse(self)

    def create(
        self,
        script_name: str,
        *,
        account_id: str,
        metadata: version_create_params.Metadata,
        bindings_inherit: Literal["strict"] | Omit = omit,
        files: SequenceNotStr[FileTypes] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionCreateResponse:
        """Upload a Worker Version without deploying to Cloudflare's network.

        You can find
        more about the multipart metadata on our docs:
        https://developers.cloudflare.com/workers/configuration/multipart-upload-metadata/.

        Args:
          account_id: Identifier.

          script_name: Name of the script.

          metadata: JSON-encoded metadata about the uploaded parts and Worker configuration.

          bindings_inherit: When set to "strict", the upload will fail if any `inherit` type bindings cannot
              be resolved against the previous version of the Worker. Without this,
              unresolvable inherit bindings are silently dropped.

          files: An array of modules (often JavaScript files) comprising a Worker script. At
              least one module must be present and referenced in the metadata as `main_module`
              or `body_part` by filename.<br/>Possible Content-Type(s) are:
              `application/javascript+module`, `text/javascript+module`,
              `application/javascript`, `text/javascript`, `text/x-python`,
              `text/x-python-requirement`, `application/wasm`, `text/plain`,
              `application/octet-stream`, `application/source-map`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        body = deepcopy_minimal(
            {
                "metadata": metadata,
            }
        )
        extracted_files = [("files", file) for file in files] if is_given(files) else []

        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/versions",
                account_id=account_id,
                script_name=script_name,
            ),
            body=maybe_transform(body, version_create_params.VersionCreateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                multipart_syntax="json",
                query=maybe_transform(
                    {"bindings_inherit": bindings_inherit}, version_create_params.VersionCreateParams
                ),
                post_parser=ResultWrapper[VersionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionCreateResponse], ResultWrapper[VersionCreateResponse]),
        )

    def list(
        self,
        script_name: str,
        *,
        account_id: str,
        deployable: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePagination[VersionListResponse]:
        """List of Worker Versions.

        The first version in the list is the latest version.

        Args:
          account_id: Identifier.

          script_name: Name of the script.

          deployable: Only return versions that can be used in a deployment. Ignores pagination.

          page: Current page.

          per_page: Items per-page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/versions",
                account_id=account_id,
                script_name=script_name,
            ),
            page=SyncV4PagePagination[VersionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "deployable": deployable,
                        "page": page,
                        "per_page": per_page,
                    },
                    version_list_params.VersionListParams,
                ),
            ),
            model=VersionListResponse,
        )

    def get(
        self,
        version_id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGetResponse:
        """
        Retrieves detailed information about a specific version of a Workers script.

        Args:
          account_id: Identifier.

          script_name: Name of the script.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/versions/{version_id}",
                account_id=account_id,
                script_name=script_name,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGetResponse], ResultWrapper[VersionGetResponse]),
        )


class AsyncVersionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncVersionsResourceWithStreamingResponse(self)

    async def create(
        self,
        script_name: str,
        *,
        account_id: str,
        metadata: version_create_params.Metadata,
        bindings_inherit: Literal["strict"] | Omit = omit,
        files: SequenceNotStr[FileTypes] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionCreateResponse:
        """Upload a Worker Version without deploying to Cloudflare's network.

        You can find
        more about the multipart metadata on our docs:
        https://developers.cloudflare.com/workers/configuration/multipart-upload-metadata/.

        Args:
          account_id: Identifier.

          script_name: Name of the script.

          metadata: JSON-encoded metadata about the uploaded parts and Worker configuration.

          bindings_inherit: When set to "strict", the upload will fail if any `inherit` type bindings cannot
              be resolved against the previous version of the Worker. Without this,
              unresolvable inherit bindings are silently dropped.

          files: An array of modules (often JavaScript files) comprising a Worker script. At
              least one module must be present and referenced in the metadata as `main_module`
              or `body_part` by filename.<br/>Possible Content-Type(s) are:
              `application/javascript+module`, `text/javascript+module`,
              `application/javascript`, `text/javascript`, `text/x-python`,
              `text/x-python-requirement`, `application/wasm`, `text/plain`,
              `application/octet-stream`, `application/source-map`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        body = deepcopy_minimal(
            {
                "metadata": metadata,
            }
        )
        extracted_files = [("files", file) for file in files] if is_given(files) else []
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/versions",
                account_id=account_id,
                script_name=script_name,
            ),
            body=await async_maybe_transform(body, version_create_params.VersionCreateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                multipart_syntax="json",
                query=await async_maybe_transform(
                    {"bindings_inherit": bindings_inherit}, version_create_params.VersionCreateParams
                ),
                post_parser=ResultWrapper[VersionCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionCreateResponse], ResultWrapper[VersionCreateResponse]),
        )

    def list(
        self,
        script_name: str,
        *,
        account_id: str,
        deployable: bool | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VersionListResponse, AsyncV4PagePagination[VersionListResponse]]:
        """List of Worker Versions.

        The first version in the list is the latest version.

        Args:
          account_id: Identifier.

          script_name: Name of the script.

          deployable: Only return versions that can be used in a deployment. Ignores pagination.

          page: Current page.

          per_page: Items per-page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/versions",
                account_id=account_id,
                script_name=script_name,
            ),
            page=AsyncV4PagePagination[VersionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "deployable": deployable,
                        "page": page,
                        "per_page": per_page,
                    },
                    version_list_params.VersionListParams,
                ),
            ),
            model=VersionListResponse,
        )

    async def get(
        self,
        version_id: str,
        *,
        account_id: str,
        script_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VersionGetResponse:
        """
        Retrieves detailed information about a specific version of a Workers script.

        Args:
          account_id: Identifier.

          script_name: Name of the script.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not script_name:
            raise ValueError(f"Expected a non-empty value for `script_name` but received {script_name!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/workers/scripts/{script_name}/versions/{version_id}",
                account_id=account_id,
                script_name=script_name,
                version_id=version_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[VersionGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[VersionGetResponse], ResultWrapper[VersionGetResponse]),
        )


class VersionsResourceWithRawResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.create = to_raw_response_wrapper(
            versions.create,
        )
        self.list = to_raw_response_wrapper(
            versions.list,
        )
        self.get = to_raw_response_wrapper(
            versions.get,
        )


class AsyncVersionsResourceWithRawResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.create = async_to_raw_response_wrapper(
            versions.create,
        )
        self.list = async_to_raw_response_wrapper(
            versions.list,
        )
        self.get = async_to_raw_response_wrapper(
            versions.get,
        )


class VersionsResourceWithStreamingResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.create = to_streamed_response_wrapper(
            versions.create,
        )
        self.list = to_streamed_response_wrapper(
            versions.list,
        )
        self.get = to_streamed_response_wrapper(
            versions.get,
        )


class AsyncVersionsResourceWithStreamingResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.create = async_to_streamed_response_wrapper(
            versions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            versions.list,
        )
        self.get = async_to_streamed_response_wrapper(
            versions.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["PresetGetResponse", "Data", "Paging"]


class Data(BaseModel):
    """Returned by Get All Presets route"""

    id: Optional[str] = None
    """ID of the preset"""

    created_at: Optional[datetime] = None
    """Timestamp this preset was created at"""

    name: Optional[str] = None
    """Name of the preset"""

    updated_at: Optional[datetime] = None
    """Timestamp this preset was last updated"""


class Paging(BaseModel):
    end_offset: float

    start_offset: float

    total_count: float


class PresetGetResponse(BaseModel):
    data: List[Data]

    paging: Paging

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["PortalCreateParams", "Server", "ServerUpdatedPrompt", "ServerUpdatedTool"]


class PortalCreateParams(TypedDict, total=False):
    account_id: Required[str]

    id: Required[str]
    """portal id"""

    hostname: Required[str]

    name: Required[str]

    allow_code_mode: bool
    """Allow remote code execution in Dynamic Workers (beta)"""

    description: str

    secure_web_gateway: bool
    """Route outbound MCP traffic through Zero Trust Secure Web Gateway"""

    servers: Iterable[Server]


class ServerUpdatedPrompt(TypedDict, total=False):
    name: Required[str]

    alias: str

    description: str

    enabled: bool


class ServerUpdatedTool(TypedDict, total=False):
    name: Required[str]

    alias: str

    description: str

    enabled: bool


class Server(TypedDict, total=False):
    server_id: Required[str]
    """server id"""

    default_disabled: bool

    on_behalf: bool

    updated_prompts: Iterable[ServerUpdatedPrompt]

    updated_tools: Iterable[ServerUpdatedTool]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ......_models import BaseModel

__all__ = ["LogGetResponse", "Data"]


class Data(BaseModel):
    line: str

    ts: str


class LogGetResponse(BaseModel):
    data: List[Data]

    includes_container_logs: bool

    total: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["KeyListParams"]


class KeyListParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    cursor: str
    """
    Opaque token indicating the position from which to continue when requesting the
    next set of records if the amount of list results was limited by the limit
    parameter. A valid value for the cursor can be obtained from the `cursors`
    object in the `result_info` structure.
    """

    limit: float
    """Limits the number of keys returned in the response.

    The cursor attribute may be used to iterate over the next batch of keys if there
    are more than the limit.
    """

    prefix: str
    """Filters returned keys by a name prefix.

    Exact matches and any key names that begin with the prefix will be returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.radar.post_quantum import tls_support_params
from ....types.radar.post_quantum.tls_support_response import TLSSupportResponse

__all__ = ["TLSResource", "AsyncTLSResource"]


class TLSResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TLSResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TLSResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TLSResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TLSResourceWithStreamingResponse(self)

    def support(
        self,
        *,
        host: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TLSSupportResponse:
        """
        Tests whether a hostname or IP address supports Post-Quantum (PQ) TLS key
        exchange. Returns information about the negotiated key exchange algorithm,
        whether it uses PQ cryptography, and any detected TLS implementation bugs (Split
        ClientHello, HRR failure, etc.).

        Args:
          host: Hostname or IP address to test for Post-Quantum TLS support, optionally with
              port (defaults to 443).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/radar/post_quantum/tls/support",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"host": host}, tls_support_params.TLSSupportParams),
                post_parser=ResultWrapper[TLSSupportResponse]._unwrapper,
            ),
            cast_to=cast(Type[TLSSupportResponse], ResultWrapper[TLSSupportResponse]),
        )


class AsyncTLSResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTLSResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTLSResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTLSResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTLSResourceWithStreamingResponse(self)

    async def support(
        self,
        *,
        host: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TLSSupportResponse:
        """
        Tests whether a hostname or IP address supports Post-Quantum (PQ) TLS key
        exchange. Returns information about the negotiated key exchange algorithm,
        whether it uses PQ cryptography, and any detected TLS implementation bugs (Split
        ClientHello, HRR failure, etc.).

        Args:
          host: Hostname or IP address to test for Post-Quantum TLS support, optionally with
              port (defaults to 443).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/radar/post_quantum/tls/support",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"host": host}, tls_support_params.TLSSupportParams),
                post_parser=ResultWrapper[TLSSupportResponse]._unwrapper,
            ),
            cast_to=cast(Type[TLSSupportResponse], ResultWrapper[TLSSupportResponse]),
        )


class TLSResourceWithRawResponse:
    def __init__(self, tls: TLSResource) -> None:
        self._tls = tls

        self.support = to_raw_response_wrapper(
            tls.support,
        )


class AsyncTLSResourceWithRawResponse:
    def __init__(self, tls: AsyncTLSResource) -> None:
        self._tls = tls

        self.support = async_to_raw_response_wrapper(
            tls.support,
        )


class TLSResourceWithStreamingResponse:
    def __init__(self, tls: TLSResource) -> None:
        self._tls = tls

        self.support = to_streamed_response_wrapper(
            tls.support,
        )


class AsyncTLSResourceWithStreamingResponse:
    def __init__(self, tls: AsyncTLSResource) -> None:
        self._tls = tls

        self.support = async_to_streamed_response_wrapper(
            tls.support,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TrueClientIPHeader"]


class TrueClientIPHeader(BaseModel):
    id: Optional[Literal["true_client_ip_header"]] = None
    """Turn on or off the True-Client-IP Header feature of the Cloudflare Network app."""

    value: Optional[Literal["on", "off"]] = None
    """The status of True Client IP Header."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["IPV6NetworkParam"]


class IPV6NetworkParam(TypedDict, total=False):
    network: Required[str]
    """Specify the IPv6 address or IPv6 CIDR."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .keys import (
    KeysResource,
    AsyncKeysResource,
    KeysResourceWithRawResponse,
    AsyncKeysResourceWithRawResponse,
    KeysResourceWithStreamingResponse,
    AsyncKeysResourceWithStreamingResponse,
)
from .values import (
    ValuesResource,
    AsyncValuesResource,
    ValuesResourceWithRawResponse,
    AsyncValuesResourceWithRawResponse,
    ValuesResourceWithStreamingResponse,
    AsyncValuesResourceWithStreamingResponse,
)
from .zone_tags import (
    ZoneTagsResource,
    AsyncZoneTagsResource,
    ZoneTagsResourceWithRawResponse,
    AsyncZoneTagsResourceWithRawResponse,
    ZoneTagsResourceWithStreamingResponse,
    AsyncZoneTagsResourceWithStreamingResponse,
)
from .account_tags import (
    AccountTagsResource,
    AsyncAccountTagsResource,
    AccountTagsResourceWithRawResponse,
    AsyncAccountTagsResourceWithRawResponse,
    AccountTagsResourceWithStreamingResponse,
    AsyncAccountTagsResourceWithStreamingResponse,
)
from .resource_tagging import (
    ResourceTaggingResource,
    AsyncResourceTaggingResource,
    ResourceTaggingResourceWithRawResponse,
    AsyncResourceTaggingResourceWithRawResponse,
    ResourceTaggingResourceWithStreamingResponse,
    AsyncResourceTaggingResourceWithStreamingResponse,
)

__all__ = [
    "AccountTagsResource",
    "AsyncAccountTagsResource",
    "AccountTagsResourceWithRawResponse",
    "AsyncAccountTagsResourceWithRawResponse",
    "AccountTagsResourceWithStreamingResponse",
    "AsyncAccountTagsResourceWithStreamingResponse",
    "ZoneTagsResource",
    "AsyncZoneTagsResource",
    "ZoneTagsResourceWithRawResponse",
    "AsyncZoneTagsResourceWithRawResponse",
    "ZoneTagsResourceWithStreamingResponse",
    "AsyncZoneTagsResourceWithStreamingResponse",
    "KeysResource",
    "AsyncKeysResource",
    "KeysResourceWithRawResponse",
    "AsyncKeysResourceWithRawResponse",
    "KeysResourceWithStreamingResponse",
    "AsyncKeysResourceWithStreamingResponse",
    "ValuesResource",
    "AsyncValuesResource",
    "ValuesResourceWithRawResponse",
    "AsyncValuesResourceWithRawResponse",
    "ValuesResourceWithStreamingResponse",
    "AsyncValuesResourceWithStreamingResponse",
    "ResourceTaggingResource",
    "AsyncResourceTaggingResource",
    "ResourceTaggingResourceWithRawResponse",
    "AsyncResourceTaggingResourceWithRawResponse",
    "ResourceTaggingResourceWithStreamingResponse",
    "AsyncResourceTaggingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ItemEditParams"]


class ItemEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    burst_sensitivity: str
    """The new burst sensitivity. Optional. Must be one of 'low', 'medium', 'high'."""

    mode: str
    """The new mode for TCP Flow Protection.

    Optional. Must be one of 'enabled', 'disabled', 'monitoring'.
    """

    rate_sensitivity: str
    """The new rate sensitivity. Optional. Must be one of 'low', 'medium', 'high'."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["CustomCertificateSettingsParam"]


class CustomCertificateSettingsParam(TypedDict, total=False):
    """Specify custom certificate settings for BYO-PKI.

    This field is deprecated; use `certificate` instead.
    """

    enabled: Required[Optional[bool]]
    """
    Specify whether to enable a custom certificate authority for signing Gateway
    traffic.
    """

    id: str
    """Specify the UUID of the certificate (ID from MTLS certificate store)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["IndexListVectorsResponse", "Vector"]


class Vector(BaseModel):
    id: str
    """Identifier for a Vector"""


class IndexListVectorsResponse(BaseModel):
    count: int
    """Number of vectors returned in this response"""

    is_truncated: bool = FieldInfo(alias="isTruncated")
    """Whether there are more vectors available beyond this response"""

    total_count: int = FieldInfo(alias="totalCount")
    """Total number of vectors in the index"""

    vectors: List[Vector]
    """Array of vector items"""

    cursor_expiration_timestamp: Optional[datetime] = FieldInfo(alias="cursorExpirationTimestamp", default=None)
    """When the cursor expires as an ISO8601 string"""

    next_cursor: Optional[str] = FieldInfo(alias="nextCursor", default=None)
    """Cursor for the next page of results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CookieAttributesParam"]


class CookieAttributesParam(TypedDict, total=False):
    """Configures cookie attributes for the waiting room cookie.

    This encrypted cookie stores a user's status in the waiting room, such as queue position.
    """

    samesite: Literal["auto", "lax", "none", "strict"]
    """Configures the SameSite attribute on the waiting room cookie.

    Value `auto` will be translated to `lax` or `none` depending if **Always Use
    HTTPS** is enabled. Note that when using value `none`, the secure attribute
    cannot be set to `never`.
    """

    secure: Literal["auto", "always", "never"]
    """Configures the Secure attribute on the waiting room cookie.

    Value `always` indicates that the Secure attribute will be set in the Set-Cookie
    header, `never` indicates that the Secure attribute will not be set, and `auto`
    will set the Secure attribute depending if **Always Use HTTPS** is enabled.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_cloud_networking.catalog_syncs import prebuilt_policy_list_params
from ....types.magic_cloud_networking.catalog_syncs.prebuilt_policy_list_response import PrebuiltPolicyListResponse

__all__ = ["PrebuiltPoliciesResource", "AsyncPrebuiltPoliciesResource"]


class PrebuiltPoliciesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PrebuiltPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PrebuiltPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PrebuiltPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PrebuiltPoliciesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        destination_type: Literal["NONE", "ZERO_TRUST_LIST"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[PrebuiltPolicyListResponse]:
        """
        List prebuilt catalog sync policies (Closed Beta).

        Args:
          destination_type: Specify type of destination, omit to return all.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/catalog-syncs/prebuilt-policies", account_id=account_id),
            page=SyncSinglePage[PrebuiltPolicyListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"destination_type": destination_type}, prebuilt_policy_list_params.PrebuiltPolicyListParams
                ),
            ),
            model=PrebuiltPolicyListResponse,
        )


class AsyncPrebuiltPoliciesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPrebuiltPoliciesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPrebuiltPoliciesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPrebuiltPoliciesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPrebuiltPoliciesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        destination_type: Literal["NONE", "ZERO_TRUST_LIST"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PrebuiltPolicyListResponse, AsyncSinglePage[PrebuiltPolicyListResponse]]:
        """
        List prebuilt catalog sync policies (Closed Beta).

        Args:
          destination_type: Specify type of destination, omit to return all.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/magic/cloud/catalog-syncs/prebuilt-policies", account_id=account_id),
            page=AsyncSinglePage[PrebuiltPolicyListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"destination_type": destination_type}, prebuilt_policy_list_params.PrebuiltPolicyListParams
                ),
            ),
            model=PrebuiltPolicyListResponse,
        )


class PrebuiltPoliciesResourceWithRawResponse:
    def __init__(self, prebuilt_policies: PrebuiltPoliciesResource) -> None:
        self._prebuilt_policies = prebuilt_policies

        self.list = to_raw_response_wrapper(
            prebuilt_policies.list,
        )


class AsyncPrebuiltPoliciesResourceWithRawResponse:
    def __init__(self, prebuilt_policies: AsyncPrebuiltPoliciesResource) -> None:
        self._prebuilt_policies = prebuilt_policies

        self.list = async_to_raw_response_wrapper(
            prebuilt_policies.list,
        )


class PrebuiltPoliciesResourceWithStreamingResponse:
    def __init__(self, prebuilt_policies: PrebuiltPoliciesResource) -> None:
        self._prebuilt_policies = prebuilt_policies

        self.list = to_streamed_response_wrapper(
            prebuilt_policies.list,
        )


class AsyncPrebuiltPoliciesResourceWithStreamingResponse:
    def __init__(self, prebuilt_policies: AsyncPrebuiltPoliciesResource) -> None:
        self._prebuilt_policies = prebuilt_policies

        self.list = async_to_streamed_response_wrapper(
            prebuilt_policies.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tls_support_params import TLSSupportParams as TLSSupportParams
from .tls_support_response import TLSSupportResponse as TLSSupportResponse
from .origin_summary_params import OriginSummaryParams as OriginSummaryParams
from .origin_summary_response import OriginSummaryResponse as OriginSummaryResponse
from .origin_timeseries_groups_params import OriginTimeseriesGroupsParams as OriginTimeseriesGroupsParams
from .origin_timeseries_groups_response import OriginTimeseriesGroupsResponse as OriginTimeseriesGroupsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["TrustedDomainListResponse"]


class TrustedDomainListResponse(BaseModel):
    """A trusted email domain"""

    id: Optional[str] = None
    """Trusted domain identifier"""

    comments: Optional[str] = None

    created_at: Optional[datetime] = None

    is_recent: Optional[bool] = None
    """
    Select to prevent recently registered domains from triggering a Suspicious or
    Malicious disposition.
    """

    is_regex: Optional[bool] = None

    is_similarity: Optional[bool] = None
    """
    Select for partner or other approved domains that have similar spelling to your
    connected domains. Prevents listed domains from triggering a Spoof disposition.
    """

    last_modified: Optional[datetime] = None
    """Deprecated, use `modified_at` instead. End of life: November 1, 2026."""

    modified_at: Optional[datetime] = None

    pattern: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .associations import (
    AssociationsResource,
    AsyncAssociationsResource,
    AssociationsResourceWithRawResponse,
    AsyncAssociationsResourceWithRawResponse,
    AssociationsResourceWithStreamingResponse,
    AsyncAssociationsResourceWithStreamingResponse,
)
from .mtls_certificates import (
    MTLSCertificatesResource,
    AsyncMTLSCertificatesResource,
    MTLSCertificatesResourceWithRawResponse,
    AsyncMTLSCertificatesResourceWithRawResponse,
    MTLSCertificatesResourceWithStreamingResponse,
    AsyncMTLSCertificatesResourceWithStreamingResponse,
)

__all__ = [
    "AssociationsResource",
    "AsyncAssociationsResource",
    "AssociationsResourceWithRawResponse",
    "AsyncAssociationsResourceWithRawResponse",
    "AssociationsResourceWithStreamingResponse",
    "AsyncAssociationsResourceWithStreamingResponse",
    "MTLSCertificatesResource",
    "AsyncMTLSCertificatesResource",
    "MTLSCertificatesResourceWithRawResponse",
    "AsyncMTLSCertificatesResourceWithRawResponse",
    "MTLSCertificatesResourceWithStreamingResponse",
    "AsyncMTLSCertificatesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .config import (
    ConfigResource,
    AsyncConfigResource,
    ConfigResourceWithRawResponse,
    AsyncConfigResourceWithRawResponse,
    ConfigResourceWithStreamingResponse,
    AsyncConfigResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["GoogleTagGatewayResource", "AsyncGoogleTagGatewayResource"]


class GoogleTagGatewayResource(SyncAPIResource):
    @cached_property
    def config(self) -> ConfigResource:
        return ConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> GoogleTagGatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return GoogleTagGatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GoogleTagGatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return GoogleTagGatewayResourceWithStreamingResponse(self)


class AsyncGoogleTagGatewayResource(AsyncAPIResource):
    @cached_property
    def config(self) -> AsyncConfigResource:
        return AsyncConfigResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncGoogleTagGatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGoogleTagGatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGoogleTagGatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncGoogleTagGatewayResourceWithStreamingResponse(self)


class GoogleTagGatewayResourceWithRawResponse:
    def __init__(self, google_tag_gateway: GoogleTagGatewayResource) -> None:
        self._google_tag_gateway = google_tag_gateway

    @cached_property
    def config(self) -> ConfigResourceWithRawResponse:
        return ConfigResourceWithRawResponse(self._google_tag_gateway.config)


class AsyncGoogleTagGatewayResourceWithRawResponse:
    def __init__(self, google_tag_gateway: AsyncGoogleTagGatewayResource) -> None:
        self._google_tag_gateway = google_tag_gateway

    @cached_property
    def config(self) -> AsyncConfigResourceWithRawResponse:
        return AsyncConfigResourceWithRawResponse(self._google_tag_gateway.config)


class GoogleTagGatewayResourceWithStreamingResponse:
    def __init__(self, google_tag_gateway: GoogleTagGatewayResource) -> None:
        self._google_tag_gateway = google_tag_gateway

    @cached_property
    def config(self) -> ConfigResourceWithStreamingResponse:
        return ConfigResourceWithStreamingResponse(self._google_tag_gateway.config)


class AsyncGoogleTagGatewayResourceWithStreamingResponse:
    def __init__(self, google_tag_gateway: AsyncGoogleTagGatewayResource) -> None:
        self._google_tag_gateway = google_tag_gateway

    @cached_property
    def config(self) -> AsyncConfigResourceWithStreamingResponse:
        return AsyncConfigResourceWithStreamingResponse(self._google_tag_gateway.config)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .summary_arc_params import SummaryARCParams as SummaryARCParams
from .summary_spf_params import SummarySPFParams as SummarySPFParams
from .summary_dkim_params import SummaryDKIMParams as SummaryDKIMParams
from .summary_arc_response import SummaryARCResponse as SummaryARCResponse
from .summary_dmarc_params import SummaryDMARCParams as SummaryDMARCParams
from .summary_spf_response import SummarySPFResponse as SummarySPFResponse
from .summary_dkim_response import SummaryDKIMResponse as SummaryDKIMResponse
from .summary_dmarc_response import SummaryDMARCResponse as SummaryDMARCResponse
from .summary_encrypted_params import SummaryEncryptedParams as SummaryEncryptedParams
from .summary_ip_version_params import SummaryIPVersionParams as SummaryIPVersionParams
from .summary_encrypted_response import SummaryEncryptedResponse as SummaryEncryptedResponse
from .summary_ip_version_response import SummaryIPVersionResponse as SummaryIPVersionResponse
from .timeseries_group_arc_params import TimeseriesGroupARCParams as TimeseriesGroupARCParams
from .timeseries_group_spf_params import TimeseriesGroupSPFParams as TimeseriesGroupSPFParams
from .timeseries_group_dkim_params import TimeseriesGroupDKIMParams as TimeseriesGroupDKIMParams
from .timeseries_group_arc_response import TimeseriesGroupARCResponse as TimeseriesGroupARCResponse
from .timeseries_group_dmarc_params import TimeseriesGroupDMARCParams as TimeseriesGroupDMARCParams
from .timeseries_group_spf_response import TimeseriesGroupSPFResponse as TimeseriesGroupSPFResponse
from .timeseries_group_dkim_response import TimeseriesGroupDKIMResponse as TimeseriesGroupDKIMResponse
from .timeseries_group_dmarc_response import TimeseriesGroupDMARCResponse as TimeseriesGroupDMARCResponse
from .timeseries_group_encrypted_params import TimeseriesGroupEncryptedParams as TimeseriesGroupEncryptedParams
from .timeseries_group_ip_version_params import TimeseriesGroupIPVersionParams as TimeseriesGroupIPVersionParams
from .timeseries_group_encrypted_response import TimeseriesGroupEncryptedResponse as TimeseriesGroupEncryptedResponse
from .timeseries_group_ip_version_response import TimeseriesGroupIPVersionResponse as TimeseriesGroupIPVersionResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .asn_ip_params import ASNIPParams as ASNIPParams
from .asn_get_params import ASNGetParams as ASNGetParams
from .asn_rel_params import ASNRelParams as ASNRelParams
from .asn_ip_response import ASNIPResponse as ASNIPResponse
from .asn_list_params import ASNListParams as ASNListParams
from .asn_get_response import ASNGetResponse as ASNGetResponse
from .asn_rel_response import ASNRelResponse as ASNRelResponse
from .asn_as_set_params import ASNAsSetParams as ASNAsSetParams
from .asn_list_response import ASNListResponse as ASNListResponse
from .asn_as_set_response import ASNAsSetResponse as ASNAsSetResponse
from .location_get_params import LocationGetParams as LocationGetParams
from .location_list_params import LocationListParams as LocationListParams
from .location_get_response import LocationGetResponse as LocationGetResponse
from .location_list_response import LocationListResponse as LocationListResponse
from .asn_botnet_threat_feed_params import ASNBotnetThreatFeedParams as ASNBotnetThreatFeedParams
from .asn_botnet_threat_feed_response import ASNBotnetThreatFeedResponse as ASNBotnetThreatFeedResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel
from .health_check_rate import HealthCheckRate
from .health_check_type import HealthCheckType

__all__ = ["HealthCheck", "Target", "TargetMagicHealthCheckTarget"]


class TargetMagicHealthCheckTarget(BaseModel):
    """The destination address in a request type health check.

    After the healthcheck is decapsulated at the customer end of the tunnel, the ICMP echo will be forwarded to this address. This field defaults to `customer_gre_endpoint address`. This field is ignored for bidirectional healthchecks as the interface_address (not assigned to the Cloudflare side of the tunnel) is used as the target.
    """

    effective: Optional[str] = None
    """The effective health check target.

    If 'saved' is empty, then this field will be populated with the calculated
    default value on GET requests. Ignored in POST, PUT, and PATCH requests.
    """

    saved: Optional[str] = None
    """The saved health check target.

    Setting the value to the empty string indicates that the calculated default
    value will be used.
    """


Target: TypeAlias = Union[TargetMagicHealthCheckTarget, str]


class HealthCheck(BaseModel):
    enabled: Optional[bool] = None
    """Determines whether to run healthchecks for a tunnel."""

    rate: Optional[HealthCheckRate] = None
    """How frequent the health check is run. The default value is `mid`."""

    target: Optional[Target] = None
    """The destination address in a request type health check.

    After the healthcheck is decapsulated at the customer end of the tunnel, the
    ICMP echo will be forwarded to this address. This field defaults to
    `customer_gre_endpoint address`. This field is ignored for bidirectional
    healthchecks as the interface_address (not assigned to the Cloudflare side of
    the tunnel) is used as the target. Must be in object form if the
    x-magic-new-hc-target header is set to true and string form if
    x-magic-new-hc-target is absent or set to false.
    """

    type: Optional[HealthCheckType] = None
    """The type of healthcheck to run, reply or request. The default value is `reply`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["DirectUploadCreateParams"]


class DirectUploadCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    id: str
    """Optional Image Custom ID.

    Up to 1024 chars. Can include any number of subpaths, and utf8 characters.
    Cannot start nor end with a / (forward slash). Cannot be a UUID.
    """

    creator: str
    """Can set the creator field with an internal user ID."""

    expiry: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """The date after which the upload will not be accepted.

    Minimum: Now + 2 minutes. Maximum: Now + 6 hours.
    """

    metadata: object
    """User modifiable key-value store.

    Can be used for keeping references to another system of record, for managing
    images.
    """

    require_signed_urls: Annotated[bool, PropertyInfo(alias="requireSignedURLs")]
    """Indicates whether the image requires a signature token to be accessed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DomainRuleParam", "EmailDomain"]


class EmailDomain(TypedDict, total=False):
    domain: Required[str]
    """The email domain to match."""


class DomainRuleParam(TypedDict, total=False):
    """Match an entire email domain."""

    email_domain: Required[EmailDomain]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ai_run_params import AIRunParams as AIRunParams
from .ai_run_response import AIRunResponse as AIRunResponse
from .model_list_params import ModelListParams as ModelListParams
from .finetune_create_params import FinetuneCreateParams as FinetuneCreateParams
from .finetune_list_response import FinetuneListResponse as FinetuneListResponse
from .finetune_create_response import FinetuneCreateResponse as FinetuneCreateResponse
from .to_markdown_transform_params import ToMarkdownTransformParams as ToMarkdownTransformParams
from .to_markdown_supported_response import ToMarkdownSupportedResponse as ToMarkdownSupportedResponse
from .to_markdown_transform_response import ToMarkdownTransformResponse as ToMarkdownTransformResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .certificate_update_params import CertificateUpdateParams as CertificateUpdateParams
from .certificate_delete_response import CertificateDeleteResponse as CertificateDeleteResponse
from .certificate_update_response import CertificateUpdateResponse as CertificateUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .phishguard import (
    PhishguardResource,
    AsyncPhishguardResource,
    PhishguardResourceWithRawResponse,
    AsyncPhishguardResourceWithRawResponse,
    PhishguardResourceWithStreamingResponse,
    AsyncPhishguardResourceWithStreamingResponse,
)
from .investigate import (
    InvestigateResource,
    AsyncInvestigateResource,
    InvestigateResourceWithRawResponse,
    AsyncInvestigateResourceWithRawResponse,
    InvestigateResourceWithStreamingResponse,
    AsyncInvestigateResourceWithStreamingResponse,
)
from .submissions import (
    SubmissionsResource,
    AsyncSubmissionsResource,
    SubmissionsResourceWithRawResponse,
    AsyncSubmissionsResourceWithRawResponse,
    SubmissionsResourceWithStreamingResponse,
    AsyncSubmissionsResourceWithStreamingResponse,
)
from .email_security import (
    EmailSecurityResource,
    AsyncEmailSecurityResource,
    EmailSecurityResourceWithRawResponse,
    AsyncEmailSecurityResourceWithRawResponse,
    EmailSecurityResourceWithStreamingResponse,
    AsyncEmailSecurityResourceWithStreamingResponse,
)

__all__ = [
    "InvestigateResource",
    "AsyncInvestigateResource",
    "InvestigateResourceWithRawResponse",
    "AsyncInvestigateResourceWithRawResponse",
    "InvestigateResourceWithStreamingResponse",
    "AsyncInvestigateResourceWithStreamingResponse",
    "PhishguardResource",
    "AsyncPhishguardResource",
    "PhishguardResourceWithRawResponse",
    "AsyncPhishguardResourceWithRawResponse",
    "PhishguardResourceWithStreamingResponse",
    "AsyncPhishguardResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "SubmissionsResource",
    "AsyncSubmissionsResource",
    "SubmissionsResourceWithRawResponse",
    "AsyncSubmissionsResourceWithRawResponse",
    "SubmissionsResourceWithStreamingResponse",
    "AsyncSubmissionsResourceWithStreamingResponse",
    "EmailSecurityResource",
    "AsyncEmailSecurityResource",
    "EmailSecurityResourceWithRawResponse",
    "AsyncEmailSecurityResourceWithRawResponse",
    "EmailSecurityResourceWithStreamingResponse",
    "AsyncEmailSecurityResourceWithStreamingResponse",
]
